﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using MVR.FileManagementSecure;
using UnityEngine.UI;
using MeshVR;


namespace JayJayWon
{

    public class JSONStorableStringChooserCustomDisplayVal : JSONStorableStringChooser
    {
        public JSONStorableStringChooserCustomDisplayVal(string paramName,List<string> choicesList,string startingValue,string displayName) : base(paramName, choicesList, startingValue, displayName)
        {

        }
        public JSONStorableStringChooserCustomDisplayVal(string paramName,List<string> choicesList, string startingValue,string displayName, JSONStorableStringChooser.SetStringCallback callback) : base(paramName, choicesList, startingValue, displayName, callback)
        {

        }
        public JSONStorableStringChooserCustomDisplayVal( string paramName, List<string> choicesList, string startingValue, string displayName,JSONStorableStringChooser.SetJSONStringCallback callback) : base(paramName, choicesList, startingValue, displayName, callback)
        {

        }

        public JSONStorableStringChooserCustomDisplayVal( string paramName,List<string> choicesList, List<string> displayChoicesList,string startingValue,string displayName) : base(paramName, choicesList, displayChoicesList, startingValue, displayName)
        {

        }

        public JSONStorableStringChooserCustomDisplayVal(string paramName,List<string> choicesList,List<string> displayChoicesList,string startingValue,string displayName,JSONStorableStringChooser.SetStringCallback callback) : base(paramName, choicesList, displayChoicesList, startingValue, displayName,callback)
        {

        }
        public JSONStorableStringChooserCustomDisplayVal( string paramName, List<string> choicesList, List<string> displayChoicesList,  string startingValue, string displayName, JSONStorableStringChooser.SetJSONStringCallback callback) : base(paramName, choicesList, displayChoicesList, startingValue, displayName, callback)
        {

        }
        public virtual new List<string> displayChoices
        {
            get
            {
                return this._displayChoices;
            }
            set
            {
                if (this._displayChoices == value)
                    return;
                this._displayChoices = value;
                this.useDifferentDisplayChoices = true;
                this.SyncPopup();
                this.SyncPopupAlt();
            }
        }

        public bool uniqueDisplayChoices { get; set; }
    }

    
    public class JSONStorableFixedIndexStringChoooser : JSONStorableStringChooserCustomDisplayVal
    {
        public JSONStorableFixedIndexStringChoooser(string paramName, List<string> fixedIndexChoicesList, List<string> dynamicIndexChoicesList, string startingValue, string displayName, SetStringCallback fixedIndexCallback, SetStringCallback dynamicIndexCallback) : base(paramName, null, startingValue, displayName, fixedIndexCallback)
        {
            setCallbackFunction = ValueChangeCallback;

            _fixedIndexChoices = fixedIndexChoicesList;
            _dynamicIndexChoices = dynamicIndexChoicesList;
            _fixedIndexCallback = fixedIndexCallback;
            _dynamicIndexCallback = dynamicIndexCallback;
            valNoCallback = "0";
            uniqueDisplayChoices = false;
            RefreshChoiceLists();
  //          SetIndexChoices(_fixedIndexChoices, _dynamicIndexChoices);
        }

        public JSONStorableFixedIndexStringChoooser(string paramName, List<string> fixedIndexChoicesList, List<string> dynamicIndexChoicesList, string startingValue, string displayName, SetJSONStringCallback fixedIndexJSONCallback, SetJSONStringCallback dynamicIndexJSONCallback) : base(paramName, null, startingValue, displayName, fixedIndexJSONCallback)
        {
            setJSONCallbackFunction = ValueChangeJSONCallback;

            _fixedIndexChoices = fixedIndexChoicesList;
            _dynamicIndexChoices = dynamicIndexChoicesList;
            _fixedIndexJSONCallback = fixedIndexJSONCallback;
            _dynamicIndexJSONCallback = dynamicIndexJSONCallback;
            valNoCallback = "0";
            uniqueDisplayChoices = false;
            RefreshChoiceLists();
  //         SetIndexChoices(_fixedIndexChoices, _dynamicIndexChoices);
        }

        protected void RefreshChoiceLists()
        {
            int fixedIndexChoicesCount = 0;
            int dynamicIndexChoicesCount = 0;
            if (_fixedIndexChoices != null) fixedIndexChoicesCount = _fixedIndexChoices.Count;
            if (_dynamicIndexChoices != null) dynamicIndexChoicesCount = _dynamicIndexChoices.Count;

            int choicesCount = fixedIndexChoicesCount + dynamicIndexChoicesCount;
            List<string> choicesList = new List<string>();
            List<string> displayChoicesList = new List<string>(choicesCount);

            if (fixedIndexChoicesCount>0) displayChoicesList.AddRange(_fixedIndexChoices);
            if (dynamicIndexChoicesCount>0) displayChoicesList.AddRange(_dynamicIndexChoices);

            for (int i = 0; i < choicesCount; i++)
            {
                choicesList.Add(i.ToString());
            }

            this.displayChoices = displayChoicesList;
            this.choices = choicesList;
        }

        protected void ValueChangeCallback(string val)
        {
            if (isValFixedIndex) _fixedIndexCallback.Invoke(displayChoices[int.Parse(val)]);
            else _dynamicIndexCallback.Invoke(displayChoices[int.Parse(val)]);
        }

        protected void ValueChangeJSONCallback(JSONStorableStringChooser val)
        {
            if (isValFixedIndex) _fixedIndexJSONCallback.Invoke(this);
            else _dynamicIndexJSONCallback.Invoke(this);
        }

        protected SetStringCallback _fixedIndexCallback;
        protected SetStringCallback _dynamicIndexCallback;
        protected SetJSONStringCallback _fixedIndexJSONCallback;
        protected SetJSONStringCallback _dynamicIndexJSONCallback;
        protected List<string> _fixedIndexChoices;
        protected List<string> _dynamicIndexChoices;

        public bool isValFixedIndex
        {
            get
            {
                return (_fixedIndexChoices!=null && int.Parse(val) < _fixedIndexChoices.Count);
            }
        }

        public string displayVal
        {
            get
            {
                int result=0;
                if (!int.TryParse(val, out result) || displayChoices.Count<1) return ("");
                return (displayChoices[result]);
            }
        }

        public void SetIndexChoices(List<string> fixedIndexChoices, List<string> dynamicIndexChoices)
        {
            int newIndex = 0;
            if (choices.Count != 0)
            {
                newIndex = int.Parse(val);
                if (isValFixedIndex && fixedIndexChoices.Contains(displayVal))
                {
                    newIndex = fixedIndexChoices.IndexOf(displayVal);
                }
                else if (!isValFixedIndex && dynamicIndexChoices.Contains(displayVal))
                {
                    newIndex = fixedIndexChoices.Count + dynamicIndexChoices.IndexOf(displayVal);
                }
                else if (newIndex >= fixedIndexChoices.Count + dynamicIndexChoices.Count) newIndex = fixedIndexChoices.Count + dynamicIndexChoices.Count - 1;
            }
            _dynamicIndexChoices = dynamicIndexChoices;
            _fixedIndexChoices = fixedIndexChoices;

            RefreshChoiceLists();
            valNoCallback = newIndex.ToString();
        }
    }

    public class JSONStorableCreatorChooser : JSONStorableFixedIndexStringChoooser
    {
        protected List<string> actualDisplayChoicesList;
        private PluginManifest pm;
        private Settings settings;
        public JSONStorableCreatorChooser(string paramName, List<string> fixedIndexChoicesList, List<string> dynamicIndexChoicesList, string startingValue, string displayName, SetStringCallback fixedIndexCallback, SetStringCallback dynamicIndexCallback, PluginAssist _pa) : base(paramName, fixedIndexChoicesList, dynamicIndexChoicesList, startingValue, displayName, fixedIndexCallback, dynamicIndexCallback)
        {
            pm = _pa.pluginManifest;
            settings = _pa.settings;
            uniqueDisplayChoices = true;
        }

        public JSONStorableCreatorChooser(string paramName, List<string> fixedIndexChoicesList, List<string> dynamicIndexChoicesList, string startingValue, string displayName, SetJSONStringCallback fixedIndexJSONCallback, SetJSONStringCallback dynamicIndexJSONCallback, PluginAssist _pa) : base(paramName, fixedIndexChoicesList, dynamicIndexChoicesList, startingValue, displayName, fixedIndexJSONCallback, dynamicIndexJSONCallback)
        {
            pm = _pa.pluginManifest;
            settings = _pa.settings;
            uniqueDisplayChoices = true;
        }

        public override List<string> displayChoices
        {
            get
            {
                return actualDisplayChoicesList;
            }
            set
            {
                actualDisplayChoicesList = value;
                List<string> displayChoicesWithTags = new List<string>();

                for (int i = 0; i < value.Count; i++)
                {
                    if (i < _fixedIndexChoices.Count) displayChoicesWithTags.Add(value[i]);
                    else
                    {
                        bool concealed = pm.GetCreatorConcealed(value[i]);
                        bool preferred = pm.GetCreatorPreferred(value[i]);

                        if (settings.concealedFilterItemsTagEnabled.val && concealed && !preferred) displayChoicesWithTags.Add("(" + value[i] + ")");
                        else if (settings.preferredFilterItemsTagEnabled.val && preferred && !concealed) displayChoicesWithTags.Add("*" + value[i] + "*");
                        else if (settings.preferredFilterItemsTagEnabled.val && settings.concealedFilterItemsTagEnabled.val && preferred && concealed) displayChoicesWithTags.Add("(*" + value[i] + "*)");
                        else displayChoicesWithTags.Add(value[i]);
                    }
                }

                if (this._displayChoices == displayChoicesWithTags)
                    return;
                this._displayChoices = displayChoicesWithTags;
                this.useDifferentDisplayChoices = true;
                this.SyncPopup();
                this.SyncPopupAlt();
            }
        }
    }

    public class JSONStorablePluginChooser : JSONStorableStringChooserCustomDisplayVal
    {
        protected List<string> actualDisplayChoicesList;
        private PluginManifest pm;
        private Settings settings;
        public JSONStorablePluginChooser(string paramName, List<string> choices, string startingValue, string displayName, SetStringCallback callback, PluginAssist _pa) : base(paramName, choices, startingValue, displayName, callback)
        {
            pm = _pa.pluginManifest;
            settings = _pa.settings;
        }

        public override List<string> displayChoices
        {
            get
            {
                return actualDisplayChoicesList;
            }
            set
            {
                actualDisplayChoicesList = value;
                List<string> displayChoicesWithTags = new List<string>();

                for (int i = 0; i < value.Count; i++)
                {
                    PluginManifestEntry pme = pm.GetPluginManifestEntry(choices[i]);
                    bool concealed = pme.allVariantsConcealed;
                    bool preferred = pme.anyVariantsPreferred;

                    if (settings.concealedFilterItemsTagEnabled.val && concealed && !preferred) displayChoicesWithTags.Add("(" + value[i] + ")");
                    else if (settings.preferredFilterItemsTagEnabled.val && preferred && !concealed) displayChoicesWithTags.Add("*" + value[i] + "*");
                    else if (settings.preferredFilterItemsTagEnabled.val && settings.concealedFilterItemsTagEnabled.val && preferred && concealed) displayChoicesWithTags.Add("(*" + value[i] + "*)");
                    else displayChoicesWithTags.Add(value[i]);

                }

                if (this._displayChoices == displayChoicesWithTags)
                    return;
                this._displayChoices = displayChoicesWithTags;
                this.useDifferentDisplayChoices = true;
                this.SyncPopup();
                this.SyncPopupAlt();
            }
        }
    }

    public class JSONStorablePathVariantChooser : JSONStorableStringChooserCustomDisplayVal
    {
        protected List<string> actualDisplayChoicesList;
        private PluginManifest pm;
        private Settings settings;
        private JSONStorablePluginChooser pluginChooser;
        public JSONStorablePathVariantChooser(string paramName, List<string> choices, string startingValue, string displayName, SetStringCallback callback, PluginAssist _pa, JSONStorablePluginChooser _pluginChooser) : base(paramName, choices, startingValue, displayName, callback)
        {
            pm = _pa.pluginManifest;
            settings = _pa.settings;
            pluginChooser = _pluginChooser;
        }

        public override List<string> displayChoices
        {
            get
            {
                return actualDisplayChoicesList;
            }
            set
            {
                actualDisplayChoicesList = value;
                List<string> displayChoicesWithTags = new List<string>();
                PluginManifestEntry pme = pm.GetPluginManifestEntry(pluginChooser.val);

                for (int i = 0; i < value.Count; i++)
                {                   
                    bool concealed = pme.GetVariantConcelealed(choices[i]);
                    bool preferred = pme.GetVariantPreferred(choices[i]);

                    if (settings.concealedFilterItemsTagEnabled.val && concealed && !preferred) displayChoicesWithTags.Add("(" + value[i] + ")");
                    else if (settings.preferredFilterItemsTagEnabled.val && preferred && !concealed) displayChoicesWithTags.Add("*" + value[i] + "*");
                    else if (settings.preferredFilterItemsTagEnabled.val && settings.concealedFilterItemsTagEnabled.val && preferred && concealed) displayChoicesWithTags.Add("(*" + value[i] + "*)");
                    else displayChoicesWithTags.Add(value[i]);

                }

                if (this._displayChoices == displayChoicesWithTags)
                    return;
                this._displayChoices = displayChoicesWithTags;
                this.useDifferentDisplayChoices = true;
                this.SyncPopup();
                this.SyncPopupAlt();
            }
        }
    }

    public class JSONStorableTagChooser : JSONStorableStringChooserCustomDisplayVal
    {
        protected List<string> actualDisplayChoicesList;
        private TagsManifest tagsManifest;
        private PluginManifest pluginManifest;
        private JSONStorableString pmeNameJSON;
        public JSONStorableTagChooser(string paramName, List<string> choices, string startingValue, string displayName, SetStringCallback callback, JSONStorableString _pmeName, MVRScript _mvrs) : base(paramName, choices, startingValue, displayName, callback)
        {
            PluginAssist pa = (PluginAssist)_mvrs;

            pmeNameJSON = _pmeName;
            tagsManifest = pa.tagsManifest;
            pluginManifest = pa.pluginManifest;

        }

        public override List<string> displayChoices
        {
            get
            {
                return actualDisplayChoicesList;
            }
            set
            {
                actualDisplayChoicesList = value;
                List<string> displayChoicesWithTags = new List<string>();

                for (int i = 0; i < value.Count; i++)
                {
                    PluginTag pluginTag = tagsManifest.GetTag(choices[i]);
                    PluginManifestEntry pme = pluginManifest.GetPluginManifestEntry(pmeNameJSON.val);
                    bool tagEnabled = pme.pluginTagSet.IsTagEnabled(pluginTag);

                    if (tagEnabled) displayChoicesWithTags.Add("[X] "+value[i]);
                    else displayChoicesWithTags.Add("[  ] " + value[i]);
                }

                if (this._displayChoices == displayChoicesWithTags)
                    return;
                this._displayChoices = displayChoicesWithTags;
                this.useDifferentDisplayChoices = true;
                this.SyncPopup();
                this.SyncPopupAlt();
            }
        }
    }

}
