﻿using MVR.FileManagementSecure;
using SimpleJSON;
using System;
using System.Collections.Generic;
using System.Linq;

namespace JayJayWon
{
    public class ScenePluginManifest
    {
        private Dictionary<string, ScenePluginManifestEntry> scenePluginManinfestDict;
        private PluginAssist pluginAssist;


        public ScenePluginManifest(PluginAssist _pluginAssist)
        {
            scenePluginManinfestDict = new Dictionary<string, ScenePluginManifestEntry>();
            pluginAssist = _pluginAssist;
            RefreshManifest();
        }

        public ScenePluginManifestEntry GetScenePluginManifestEntry(string pmeName)
        {
            if (scenePluginManinfestDict.ContainsKey(pmeName)) return (scenePluginManinfestDict[pmeName]);
            return null;
        }

        public List<string> GetPluginsFromScene(bool pmeName = false)
        {
            List<string> pluginsList = new List<string>();
            if (pmeName) pluginsList = scenePluginManinfestDict.Values.Select(p => p.pluginManifestEntryName).ToList();
            else pluginsList = scenePluginManinfestDict.Values.Select(p => p.pluginName).ToList();            

            return (pluginsList);
        }

        public void RefreshManifest()
        {
            scenePluginManinfestDict.Clear();
            MVRPluginManager manager;
            foreach (Atom atom in SuperController.singleton.GetAtoms())
            {
                manager = atom.GetStorableByID("PluginManager") as MVRPluginManager;
                RefreshManifestForAtom(atom, manager);
            }
            manager = pluginAssist.manager;
            RefreshManifestForAtom(pluginAssist.containingAtom, manager);
        }
        public void RemoveAtomFromManifest(Atom atom)
        {
            List<string> keysToRemove = new List<string>();
            foreach(KeyValuePair<string,ScenePluginManifestEntry> kvp in scenePluginManinfestDict)
            {
                kvp.Value.RemoveAtom(atom) ;
                if (kvp.Value.GetAtomList().Count == 0) keysToRemove.Add(kvp.Key);                   
            }
            foreach(string key in keysToRemove) scenePluginManinfestDict.Remove(key);
        }

        public void RefreshManifestForAtom(Atom atom, MVRPluginManager manager)
        {         
            if (manager != null)
            {
                JSONClass managerJS = manager.GetJSON(true, true, true);
                JSONClass currentPluginsJS = (JSONClass)managerJS["plugins"];
                if (currentPluginsJS != null)
                {
                    foreach (KeyValuePair<string, JSONNode> kvp in currentPluginsJS)
                    {
                        string pluginEntry = currentPluginsJS[kvp.Key];
                        if (pluginEntry != "")
                        {
                            bool isLocal = false;
                            string creatorName;
                            string packageName = "";
                            string pluginName;
                            string pathWithPluginName;
                            string scenePMEName;

                            if (pluginEntry.Contains(":"))
                            {
                                string varName = pluginEntry.Substring(0, pluginEntry.IndexOf(':'));
                                pathWithPluginName = pluginEntry.Substring(pluginEntry.IndexOf(':') + 2);
                                creatorName = varName.Substring(0, pluginEntry.IndexOf('.'));
                                packageName = varName.Substring(creatorName.Length + 1, varName.IndexOf('.', creatorName.Length + 1) - (creatorName.Length + 1));
                            }
                            else
                            {
                                creatorName = "LOCAL";
                                pathWithPluginName = pluginEntry;
                                isLocal = true;
                            }
                            string pluginFileName = pathWithPluginName.Substring(pathWithPluginName.LastIndexOf('/') + 1);

                            pluginName = pluginFileName.Substring(0, pluginFileName.LastIndexOf('.'));

                            if (isLocal) scenePMEName = creatorName + "." + pluginName;
                            else scenePMEName = creatorName + "." + packageName + "." + pluginName;

                            if (!scenePluginManinfestDict.ContainsKey(scenePMEName)) scenePluginManinfestDict.Add(scenePMEName, new ScenePluginManifestEntry(creatorName, pluginName, packageName, isLocal));
                            scenePluginManinfestDict[scenePMEName].AddAtomPluginSlot(atom, kvp.Key);
                            
                        }

                    }
                }
            }
        }

    }


    public class ScenePluginManifestEntry
    {
        public string creator { get; private set; }
        public string varPackageName { get; private set; }
        public string pluginName { get; private set; }
        public bool isLocal { get; private set; }

        private Dictionary<Atom, HashSet<string>> atomPluginSlotsDict;
        
        public string pluginManifestEntryName
        {
            get {
                if (isLocal) return (creator + "." + pluginName);
                return (creator + "." + varPackageName + "." + pluginName);
            }
        }

        public ScenePluginManifestEntry(string _creator, string _pluginName, string _varPackageName, bool _isLocal)
        {          
            creator = _creator;
            isLocal = _isLocal;
            varPackageName = _varPackageName;
            pluginName = _pluginName;

            atomPluginSlotsDict = new Dictionary<Atom, HashSet<string>>();

        }
        public void RemoveAtom(Atom atom)
        {
            atomPluginSlotsDict.Remove(atom);
        }

        public List<Atom> GetAtomList()
        {
            return (atomPluginSlotsDict.Keys.ToList());
        }

        public List<string> GetPluginSlotsFromAtom(Atom atom)
        {
            return (atomPluginSlotsDict[atom].ToList());
        }

        public void AddAtomPluginSlot(Atom atom, string pluginSlotRef)
        {
            if (!atomPluginSlotsDict.ContainsKey(atom)) atomPluginSlotsDict.Add(atom, new HashSet<string>());
 //           int slotNumber = int.Parse(pluginSlotRef.Substring(7));
            atomPluginSlotsDict[atom].Add(pluginSlotRef);
        }



    }


}
